
//#include <stdio.h>
#include <iostream>
//#include "iflinterface.h"
//#include "ImageWin.h"
#include <signal.h>
#pragma warning(disable : 4786)
#include <queue>

#include "opteonglue.h"  

using namespace std;

//#define USEWIN2 if (0)
#define USEWIN2 

//DEPICT_HANDLE m_hCamera;
//DEPICT_HANDLE m_hCamera2;
//void * m_pvImage[10];
//void * m_pvImage2[10];
//void *pvRGB32Image ;
//void *pvRGB32Image2 ;

//   ULONG m_ulImageWidth;
//   ULONG m_ulImageHeight;
//   ULONG ulResult;

/////////////////////
DEPICT_HANDLE gOpenOpteonCameras[4]={DEPICT_INVALID_HANDLE,DEPICT_INVALID_HANDLE,DEPICT_INVALID_HANDLE,DEPICT_INVALID_HANDLE};
std::queue<RawImage *> gPendingImageBuffers;

// We need to trap interupts and close the cameras properly because we crash otherwise.
// This interupt handler gets installed when we first open a camera
void intSigHandlerOpteon(int )
{
  puts ("Opteon interupt trap - exiting...");

  for (int i=0; i<4;i++) {

	  if ( gOpenOpteonCameras[i] != DEPICT_INVALID_HANDLE) {
		  puts ("Calling DepictClose");
		  DepictCloseDevice(gOpenOpteonCameras[i]);
		  gOpenOpteonCameras[i]=DEPICT_INVALID_HANDLE;
	  }
  }

  puts ("Opteon interupt trap - about to exit");

  exit(0);
}

// Similarly we need to trap for the program exiting, and properly close down the cameras
// We construct a global object which will be destructed at exit time, allowing us to close
// the cameras
class ExitCondition {
public:
	~ExitCondition () {
		puts ("ExitingCondition enter");
		
		for (int i=0; i<4;i++) {	
			if ( gOpenOpteonCameras[i] != DEPICT_INVALID_HANDLE) {
				puts ("Calling DepictClose");
				DepictCloseDevice(gOpenOpteonCameras[i]);
				gOpenOpteonCameras[i]=DEPICT_INVALID_HANDLE;
			}
		}
		puts ("ExitingCondition exit");
	}
};
ExitCondition gExitCondition;

ULONG vidGetNumCameras()
{
	ULONG ulNumCameras;
	
	DepictEnumDevices(DepictPlatformTypeUnknown);
	
	DepictGetNumEnumCameras(DepictPlatformTypeUnknown, &ulNumCameras);

	return ulNumCameras;
}
 
 
VidInfo *vidInit(int camNum /* =0 */ )
{
	//// Make sure that the camera closing interupt handler is installed
	signal(SIGINT, intSigHandlerOpteon);

	//// Make sure we passed a sensible camera number
	if (camNum<0 || camNum>3) {
		puts ("Invalid camera number");
		return NULL;
	}

	//// Create a new structure
	VidInfo *vid=new VidInfo;
	
	//// Save the camera number
	vid->thisCameraNumber=camNum;

	//// Try to open the camera	
	vid->m_hCamera = DEPICT_INVALID_HANDLE;

	ULONG ulNumCameras;
	
	DepictEnumDevices(DepictPlatformTypeUnknown);
	
	DepictGetNumEnumCameras(DepictPlatformTypeUnknown, &ulNumCameras);
	if(ulNumCameras == 0)
	{
		puts("Could not find a camera to open.");
		//m_oSnapButton.EnableWindow(FALSE);
		delete vid;
		return NULL;
	}
	
	ULONG ulCurCamera = 0;
	ULONG ulCameraID;
	ULONG ulPlatformID;
	ULONG ulSerialNum;
	BOOL bOpen;
	
	//for (int r=0;r<=camNum;r++) {
	//	do
	//	{
			DepictGetEnumCameraInfo(DepictPlatformTypeUnknown, 
				camNum,
				&ulPlatformID,
				&ulCameraID,
				&ulSerialNum,
				&bOpen);
			vid->m_hCamera = DepictOpenCamera(ulPlatformID, ulCameraID);
			vid->ulCameraID=ulCameraID;
			ulCurCamera++;
	//	}
	//	while(ulCurCamera < ulNumCameras && vid->m_hCamera == DEPICT_INVALID_HANDLE);
	//}
	
	

	//// If we failed return a NULL pointer
	if(vid->m_hCamera == DEPICT_INVALID_HANDLE)
		{
			puts("Could not open the desired attached camera.");
			//m_oSnapButton.EnableWindow(FALSE);
			delete vid;
			return NULL;
		}


	//// Keep track of this camera in the global array
	gOpenOpteonCameras[camNum]=vid->m_hCamera;

	//// Get the camera image size
	DepictGetCameraMaxROI(vid->m_hCamera, &(vid->m_ulImageWidth), &(vid->m_ulImageHeight));

	//// Set this camera to pipelined mode
	// DepictSetExposureToPipelined(vid->m_hCamera);

	//// Some other nonsense
	//ULONG exposure;
	//DepictGetPrimaryExposure(vid->m_hCamera, &exposure);
	//cout << "Exposure " << exposure << endl;
	DepictSetPrimaryExposure(vid->m_hCamera, 1000);
	//DepictGetPrimaryExposure(vid->m_hCamera, &exposure);
	//cout << "Exposure " << exposure << endl;

	//// Allocate buffer storage
	for (int i=0;i<10;i++) {
		vid->m_pvImage[i] = DepictCreateImageDestinationEx(vid->m_ulImageWidth,
			vid->m_ulImageHeight,
			DepictFormatRGB32);
	}
	vid->pvRGB32Image = DepictCreateImageDestinationEx(vid->m_ulImageWidth,	vid->m_ulImageHeight, DepictFormatRGB32);

	//// Return the structure
	return vid;
}

////////////////////////////////////////////////
void vidSetSlaveToMaster(VidInfo *slave, VidInfo *master)
{
	// Attach one channel to another
	DepictSetSlaveToChannel(slave->m_hCamera,master->ulCameraID);

	// Make sure we later do vidIn properly as a slave
	slave->eTriggerMode=DepictTriggerSlave;

}


////////////////////////////////////////////////
void vidSetShutterTimeMicroSec(VidInfo *vid, long ms)
{
	DepictSetPrimaryExposure(vid->m_hCamera, ms);

}


////////////////////////////////////////////////
void     vidQuit(VidInfo *info)
{

	puts ("VidQuit enter");

	if (info==NULL) return;

	// Close the camera
	if (info->m_hCamera != DEPICT_INVALID_HANDLE) {
		if (gOpenOpteonCameras[info->thisCameraNumber] == DEPICT_INVALID_HANDLE) {
			puts ("OOps");
		}
		DepictCloseDevice(info->m_hCamera);

	} else {
		puts ("Something weird! No camera to close");
	}

	// Keep the global pointer set correctly
	gOpenOpteonCameras[info->thisCameraNumber] = DEPICT_INVALID_HANDLE;

	// Delete the memory that goes with this 
	delete info;

	puts ("VidQuit exit");
}

/////////////////////////////////////////////////
void copyDepictRGBtoJDImage(void *pvRGB32Image, int m_ulImageWidth, int m_ulImageHeight, JDImage &im)
{
	// Make sure the buffer is big enough
	if (im.xsize!=m_ulImageWidth || im.ysize!=m_ulImageHeight) {
		im.resize(m_ulImageWidth, m_ulImageHeight);
	}

	unsigned char *capIm=(unsigned char *)pvRGB32Image;
	unsigned long buf;
	unsigned char *bufc= (unsigned char *)&buf;
	
	for (int y=0;y<m_ulImageHeight*m_ulImageWidth;y++) {
		bufc[0]=capIm[4*y+2];
		bufc[1]=capIm[4*y+1];
		bufc[2]=capIm[4*y+0];
		bufc[3]=255;
		im.pixels[y]=buf;
	}
	
}

////////////////////////////////////////////////
void copyDepictRGBtoJDImageDirect(void *pvRGB32Image, int m_ulImageWidth, int m_ulImageHeight, JDImage &im)
{
	// Make sure the buffer is big enough
	if (im.xsize!=m_ulImageWidth || im.ysize!=m_ulImageHeight) {
		im.resize(m_ulImageWidth, m_ulImageHeight);
	}

	memcpy(im.pixels,pvRGB32Image,m_ulImageHeight*m_ulImageWidth*4);

	
}

	
void vidIn(VidInfo *info, RawImage *im)
{

	// Get a raw image
	char *cpix=vidIn(info);

	// Copy the bytes
	memcpy(im,cpix,info->m_ulImageHeight* info->m_ulImageWidth);

}

////////////////////////////////////////////////
// Wrap up the raw capture of an image into our format
void vidIn(VidInfo *info, JDImage &im)
{
	// Make sure the buffer is big enough
	long x,y;
	vidGetXYSize(info, &x, &y);
	if (im.xsize!=x || im.ysize!=y) {
		im.resize(x,y);
	}

	// Get a raw image
	char *cpix=vidIn(info);

	// Convert the raw image to RGB
	DepictConvertBayerToRGB32(cpix, info->pvRGB32Image, 0, 0 ,x ,y);


	// Copy this into the JDImage structure
	copyDepictRGBtoJDImage(info->pvRGB32Image ,x,y,im);
}

////////////////////////////////////////////////
// Wrap up the raw capture of an image into our format
void vidIn_BGRA(VidInfo *info, JDImage &im)
{
	// Make sure the buffer is big enough
	long x,y;
	vidGetXYSize(info, &x, &y);
	if (im.xsize!=x || im.ysize!=y) {
		im.resize(x,y);
	}

	// Get a raw image
	char *cpix=vidIn(info);

	// Convert the raw image to RGB
	DepictConvertBayerToRGB32(cpix, info->pvRGB32Image, 0, 0 ,x ,y);


	// Copy this into the JDImage structure
	copyDepictRGBtoJDImageDirect(info->pvRGB32Image ,x,y,im);
}

////////////////////////////////////////////////
char    *vidIn(VidInfo *info )
{
	ULONG ulResult;
	
	info->idlecount++;			
	
	int buffer=info->count%10;
	//int prevbuffer=(info->count+10-1)%10;

	unsigned long starttime=Get_Milliseconds();
	
	info->sStatus[buffer].hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
	info->sStatus[buffer].hTriggerEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
	info->sStatus[buffer].pTriggerTimestamp = &(info->sTriggerTimestamp[buffer]);
	
	//cout << " about to acquire" << endl;
	ulResult = DepictSingleAcquireEx(info->m_hCamera, info->eTriggerMode, info->m_pvImage[buffer], &(info->sStatus[buffer]), NULL, 0);
	//ulResult = DepictWaitAcquireCompleteEx(m_hCamera, &(sStatus[buffer]), 2000);
	if(DEPICT_IS_ERROR(ulResult))
	{
		cout << "Error acquiring image. " << ulResult << endl;
		for (int i=0; i<200; i++) {
			if (ulResult == ((ULONG) DEPICT_MAKE_ERROR(i))) {
				cout << "Error code " << i << endl;
				intSigHandlerOpteon(5);
			}
		}
	}
	info->bufferQueue.push(buffer);
	//cout << "Starting " << buffer << endl;
	
	
	int waitbuffer=0;

	if (info->count>0) {
		//cout << " about to wait" << endl;
		waitbuffer=info->bufferQueue.front();
		info->bufferQueue.pop();
			
		unsigned long midtime=Get_Milliseconds();

		ulResult = DepictWaitAcquireCompleteEx(info->m_hCamera, &(info->sStatus[waitbuffer]), 2000);
		unsigned long endtime=Get_Milliseconds();

		if(DEPICT_IS_ERROR(ulResult))
		{
			cout << "Error acquiring image. " << ulResult << endl;
			for (int i=0; i<200; i++) {
				if (ulResult == ((ULONG) DEPICT_MAKE_ERROR(i))) {
					cout << "Error code " << i << endl;
					intSigHandlerOpteon(5);
				}
			}
		}
		//cout << "Waiting " << waitbuffer << " " << endtime-midtime << endl;

	}
	
	info->count++;
	unsigned long thistime=Get_Milliseconds();
	
	//if (count >20 ) intSigHandler(5);
	
	return (char *) info->m_pvImage[waitbuffer];
}

/////////////////////////////////////////

void vidSetPipelineMode(VidInfo *info)
{
	DepictSetExposureToPipelined(info->m_hCamera);

	vidStartCapture(info);
	//info->count++;
	vidStartCapture(info);
	//info->count++;
}

/////////////////////////////////////////

void vidStartCapture(VidInfo *info)
{
	ULONG ulResult;
	
	info->idlecount++;			
	
	int buffer=info->count%10;
	//int prevbuffer=(info->count+10-1)%10;
	
	unsigned long starttime=Get_Milliseconds();
	
	info->sStatus[buffer].hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
	info->sStatus[buffer].hTriggerEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
	info->sStatus[buffer].pTriggerTimestamp = &(info->sTriggerTimestamp[buffer]);
	
	//cout << " about to acquire" << endl;
	ulResult = DepictSingleAcquireEx(info->m_hCamera, info->eTriggerMode, info->m_pvImage[buffer], &(info->sStatus[buffer]), NULL, 0);
	//ulResult = DepictWaitAcquireCompleteEx(m_hCamera, &(sStatus[buffer]), 2000);
	if(DEPICT_IS_ERROR(ulResult))
	{
		cout << "Error acquiring image. " << ulResult << endl;
		for (int i=0; i<200; i++) {
			if (ulResult == ((ULONG) DEPICT_MAKE_ERROR(i))) {
				cout << "Error code " << i << endl;
				intSigHandlerOpteon(5);
			}
		}
	}
			
	unsigned long midtime=Get_Milliseconds();
	
	cout << "Starting " << buffer << endl;

	info->bufferQueue.push(buffer);
	info->count++;
}

/////////////////////////////////////////

char * vidFinishCapture(VidInfo *info)
{
	ULONG ulResult;
	
	info->idlecount++;			
	
	//int buffer=info->count%10;
	//int prevbuffer=(info->count+10-1)%10;
	
	int waitbuffer=info->bufferQueue.front();
	info->bufferQueue.pop();
	cout << "Waiting " << waitbuffer << endl;

	unsigned long starttime=Get_Milliseconds();
	

	if (info->count>0) {
		//cout << " about to wait" << endl;
		
		ulResult = DepictWaitAcquireCompleteEx(info->m_hCamera, &(info->sStatus[waitbuffer]), 2000);
		if(DEPICT_IS_ERROR(ulResult))
		{
			cout << "Error acquiring image. " << ulResult << endl;
			for (int i=0; i<200; i++) {
				if (ulResult == ((ULONG) DEPICT_MAKE_ERROR(i))) {
					cout << "Error code " << i << endl;
					intSigHandlerOpteon(5);
				}
			}
		}
		unsigned long endtime=Get_Milliseconds();
		
	}
	
	info->count++;
	unsigned long thistime=Get_Milliseconds();
	
	//if (count >20 ) intSigHandler(5);
	
	return (char *) info->m_pvImage[waitbuffer];
}

/////////////////////////////////////////
void     vidGetXYSize(VidInfo *info, long *x, long *y)
{
	*x = info->m_ulImageWidth;
	*y = info->m_ulImageHeight;
}

/////////////////////////////////////////
RawImage *AllocateRawImageSpace(VidInfo *vid)
{
	int width=vid->m_ulImageWidth;
	int height=vid->m_ulImageHeight;

	RawImage *im;
	
	if (gPendingImageBuffers.size()==0) {
		im=DepictCreateImageDestinationEx(width, height, DepictFormatGrayscale);
	} else {
		im=gPendingImageBuffers.front();
		gPendingImageBuffers.pop();
	}

	return im;
}

/////////////////////////////////////////
void DeleteRawImageSpace(RawImage *im)
{
	gPendingImageBuffers.push(im);

	//DepictFreeImageDestination(im);
}

/////////////////////////////////////////
void PreAllocateRawImageSpace(VidInfo *vid, int num)
{
	int width=vid->m_ulImageWidth;
	int height=vid->m_ulImageHeight;

	for (int i=0; i<num; i++) {
		RawImage *im=DepictCreateImageDestinationEx(width, height, DepictFormatGrayscale);
		gPendingImageBuffers.push(im);
	}
}

/////////////////////////////////////////
void convertRawImageToJDImage(VidInfo *vid, RawImage *im, JDImage &jdim)
{
	// Make sure the buffer is big enough
	long x,y;
	vidGetXYSize(vid, &x, &y);
	if (jdim.xsize!=x || jdim.ysize!=y) {
		jdim.resize(x,y);
	}

	// Get a raw image
	void *cpix=im;;

	// Get some tmp storage for conversion
	void *pvRGB32Image=DepictCreateImageDestinationEx(x,y,DepictFormatRGB32);

	// Convert the raw image to RGB
	DepictConvertBayerToRGB32(cpix, pvRGB32Image, 0, 0 ,x ,y);


	// Copy this into the JDImage structure
	copyDepictRGBtoJDImage(pvRGB32Image ,x,y,jdim);

	// Get rid of the tmp storage
	DepictFreeImageDestination(pvRGB32Image);
}

/////////////////////////////////////////
void convertRawImageToJDImageBGRA(VidInfo *vid, RawImage *im, JDImage &jdim)
{
	// Make sure the buffer is big enough
	long x,y;
	vidGetXYSize(vid, &x, &y);
	if (jdim.xsize!=x || jdim.ysize!=y) {
		jdim.resize(x,y);
	}

	// Get a raw image
	void *cpix=im;;

	// Get some tmp storage for conversion
	void *pvRGB32Image=DepictCreateImageDestinationEx(x,y,DepictFormatRGB32);

	// Convert the raw image to RGB
	DepictConvertBayerToRGB32(cpix, pvRGB32Image, 0, 0 ,x ,y);


	// Copy this into the JDImage structure
	copyDepictRGBtoJDImageDirect(pvRGB32Image ,x,y,jdim);

	// Get rid of the tmp storage
	DepictFreeImageDestination(pvRGB32Image);

  
}





/////////////////////////////////////////
/////////////////////////////////////////
/////////////////////////////////////////
/////////////////////////////////////////

///////////////////////////////////////
#ifdef DONTRUNTHIS_BELOWHERE

class VideoWin : public ImageWin {
public:
	int count;
	unsigned long basetime;
	DEPICT_HANDLE m_hCamera;
	void * m_pvImage[10];
	void *pvRGB32Image;
	DEPICT_ACQ_STATUS sStatus[10];
	LARGE_INTEGER sTriggerTimestamp[10];
	unsigned long prevendtime;
	int idlecount;

	VideoWin() {idlecount=0; count=0; basetime=Get_Milliseconds();}
		 
	void display () {
		idlecount--;
		unsigned long starttime=Get_Milliseconds();

		ImageWin::display();
		//glFlush();
		unsigned long endtime=Get_Milliseconds();
		cout << "display idlecount " << idlecount << " time " << endtime-starttime << endl;

	}

	void idle() {
		idlecount++;
		
		// Capture a new image
		/*ulResult = DepictSingleAcquireEx(m_hCamera, 
			 DepictTriggerImmediate,
			m_pvImage,
			NULL, NULL, 0);*/
			

		int buffer=count%10;
		int prevbuffer=(count+10-1)%10;
		//assert(prevbuffer>=0);
		//buffer=0; prevbuffer=0;

		
		unsigned long starttime=Get_Milliseconds();

		int numCaps=1;

		//for (int buffer=0; buffer<numCaps; buffer++) {
			
			sStatus[buffer].hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
			sStatus[buffer].hTriggerEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
			sStatus[buffer].pTriggerTimestamp = &(sTriggerTimestamp[buffer]);
			
			cout << " about to acquire" << endl;
			ulResult = DepictSingleAcquireEx(m_hCamera, DepictTriggerImmediate, m_pvImage[buffer], &sStatus[buffer], NULL, 0);
			//ulResult = DepictWaitAcquireCompleteEx(m_hCamera, &(sStatus[buffer]), 2000);
			if(DEPICT_IS_ERROR(ulResult))
			{
				cout << "Error acquiring image. " << ulResult << endl;
				for (int i=0; i<200; i++) {
					if (ulResult == ((ULONG) DEPICT_MAKE_ERROR(i))) {
						cout << "Error code " << i << endl;
						intSigHandler(5);
					}
				}
			}
			
			
		//}
		unsigned long midtime=Get_Milliseconds();

		{
		//for (int buffer=0; buffer<numCaps; buffer++) {
			
	//		ulResult = DepictWaitAcquireCompleteEx(m_hCamera, &(sStatus[buffer]), 2000);
			
			
	//	}
		}


		
		if (count>0) {
			cout << " about to wait" << endl;

			ulResult = DepictWaitAcquireCompleteEx(m_hCamera, &(sStatus[prevbuffer]), 2000);
			if(DEPICT_IS_ERROR(ulResult))
			{
				cout << "Error acquiring image. " << ulResult << endl;
				for (int i=0; i<200; i++) {
					if (ulResult == ((ULONG) DEPICT_MAKE_ERROR(i))) {
						cout << "Error code " << i << endl;
						intSigHandler(5);
					}
				}
			}
			unsigned long endtime=Get_Milliseconds();
			
			// save to file
			if (0) {
				ulResult = DepictSaveImage(m_pvImage[prevbuffer], "testingImage.bmp", m_ulImageWidth, m_ulImageHeight, true);
				
				if(DEPICT_IS_ERROR(ulResult))
				{
					cout << "Error saving image. " << ulResult << endl;
					for (int i=0; i<200; i++) {
						if (ulResult == ((ULONG) DEPICT_MAKE_ERROR(i))) {
							cout << "Error code " << i << endl;
						}
					}
				}
				
			}

			// Convert to color
			
			if (0)
			{
				DepictConvertBayerToRGB32(m_pvImage[prevbuffer], pvRGB32Image, 0,0,m_ulImageWidth,
					m_ulImageHeight);
			}
			unsigned long colortime=Get_Milliseconds();
			
			// Copy to my format
			if (0)
			{
				unsigned char *capIm=(unsigned char *)pvRGB32Image;
				unsigned long buf;
				unsigned char *bufc= (unsigned char *)&buf;
				
				for (int y=0;y<m_ulImageHeight*m_ulImageWidth;y++) {
					bufc[0]=capIm[4*y+2];
					bufc[1]=capIm[4*y+1];
					bufc[2]=capIm[4*y+0];
					bufc[3]=255;
					imageinfo.pixels[y]=buf;
				}
			}

			// Reduced copy to my format
			if (1)
			{
				unsigned char *capIm=(unsigned char *)m_pvImage[prevbuffer];
				unsigned long buf;
				unsigned char *bufc= (unsigned char *)&buf;
				
				for (int y=0;y<m_ulImageHeight*m_ulImageWidth;y=y+8) {
					unsigned char foo=capIm[y];
					bufc[0]=foo;
					bufc[1]=foo;
					bufc[2]=foo;
					bufc[3]=255;
					imageinfo.pixels[y]=buf;
				}
			}



		unsigned long jdimtime=Get_Milliseconds();

			// Display
			if (1)
			{
				glutPostRedisplay();
			}

		unsigned long posttime=Get_Milliseconds();

		cout << "Time mid: " << midtime-starttime << " end: " << endtime-starttime <<
			"color " << colortime-starttime << " jdimage " << jdimtime-starttime <<
			" post " << posttime-starttime << "last: " << endtime-prevendtime << endl;
		prevendtime=endtime;
			
		}

		count++;
		unsigned long thistime=Get_Milliseconds();

		//if (count >20 ) intSigHandler(5);

		cout << "Idle " << count << " fps  " << (float)count*1000/(thistime-basetime) << endl;

	}
};

/////////////////////////////////////////////
/////////////////////////////////////////////
/////////////////////////////////////////////

void main(int argc, char ** argv)
{
JDImage im;
JDImage im2;

	signal(SIGINT, intSigHandler);
	DEPICT_ACQ_STATUS sStatus;
	LARGE_INTEGER sTriggerTimestamp;

   // Try to open first camera
   ULONG ulNumCameras;
   
   DepictEnumDevices(DepictPlatformTypeUnknown);
   
   DepictGetNumEnumCameras(DepictPlatformTypeUnknown, &ulNumCameras);
   if(ulNumCameras == 0)
   {
      puts("Could not find a camera to open.");
      //m_oSnapButton.EnableWindow(FALSE);
      return ;
   }

   // Open the first available camera
   ULONG ulCurCamera = 0;
   ULONG ulCameraID;
   ULONG ulPlatformID;
   ULONG ulSerialNum;
   BOOL bOpen;

   do
   {
      DepictGetEnumCameraInfo(DepictPlatformTypeUnknown, 
                              ulCurCamera,
                              &ulPlatformID,
                              &ulCameraID,
                              &ulSerialNum,
                              &bOpen);
      m_hCamera = DepictOpenCamera(ulPlatformID, ulCameraID);
      ulCurCamera++;
   }
   while(ulCurCamera < ulNumCameras && m_hCamera == DEPICT_INVALID_HANDLE);
	
   if(m_hCamera == DEPICT_INVALID_HANDLE)
   {
      puts("Could not open any attached camera.");
      //m_oSnapButton.EnableWindow(FALSE);
      return ;
   }
	
	do
   {
      DepictGetEnumCameraInfo(DepictPlatformTypeUnknown, 
                              ulCurCamera,
                              &ulPlatformID,
                              &ulCameraID,
                              &ulSerialNum,
                              &bOpen);
      m_hCamera2 = DepictOpenCamera(ulPlatformID, ulCameraID);
      ulCurCamera++;
   }
   while(ulCurCamera < ulNumCameras && m_hCamera2 == DEPICT_INVALID_HANDLE);
	
   if(m_hCamera2 == DEPICT_INVALID_HANDLE)
   {
      puts("Could not open any attached camera. part 2");
      //m_oSnapButton.EnableWindow(FALSE);
      return ;
   }

   // Get max ROI
   DepictGetCameraMaxROI(m_hCamera, &m_ulImageWidth, &m_ulImageHeight);
   DepictSetPrimaryROIEx(m_hCamera, 0, 0, m_ulImageWidth, m_ulImageHeight);

	cout << "width height " << m_ulImageWidth << " " << m_ulImageHeight << endl;

   // Create image buffer
/*   m_pvImage = DepictCreateImageDestinationEx(m_ulImageWidth,
		m_ulImageHeight,
	DepictFormatGrayscale);*/
	VideoWin win;
	VideoWin *tmpwin=NULL;
	USEWIN2 tmpwin=new VideoWin;
	VideoWin &win2= *tmpwin;

	for (int i=0;i<10;i++) {
		win.m_pvImage[i] = DepictCreateImageDestinationEx(m_ulImageWidth,
			m_ulImageHeight,
			DepictFormatRGB32);
		USEWIN2 win2.m_pvImage[i] = DepictCreateImageDestinationEx(m_ulImageWidth,m_ulImageHeight,DepictFormatRGB32);
	}

	win.m_hCamera=m_hCamera;
	USEWIN2 win2.m_hCamera=m_hCamera2;

	sStatus.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
	sStatus.hTriggerEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
	sStatus.pTriggerTimestamp = &sTriggerTimestamp;

		  // Do grab

   /*ulResult = DepictSingleAcquireEx(m_hCamera, 
                                    DepictTriggerImmediate,
                                    m_pvImage,
                                    &sStatus, NULL, 0);*/
   ulResult = DepictSingleAcquireEx(win.m_hCamera, 
                                    DepictTriggerImmediate,
                                    win.m_pvImage[0],
                                    NULL, NULL, 0);

   if(DEPICT_IS_ERROR(ulResult))
   {
      cout << "Error acquiring image. " << ulResult << endl;
		for (int i=0; i<200; i++) {
			if (ulResult == ((ULONG) DEPICT_MAKE_ERROR(i))) {
				cout << "Error code " << i << endl;
			}
		}
   }
   else
   {
      //m_oView.SetBuffer(long(m_pvImage), m_ulImageWidth, m_ulImageHeight);
   }

	//ulResult = DepictWaitAcquireCompleteEx(m_hCamera, 
	
	
	ulResult = DepictSaveImage(win.m_pvImage[0], "testingImage.bmp", m_ulImageWidth, m_ulImageHeight, true);

	if(DEPICT_IS_ERROR(ulResult))
   {
      cout << "Error saving image. " << ulResult << endl;
		for (int i=0; i<200; i++) {
			if (ulResult == ((ULONG) DEPICT_MAKE_ERROR(i))) {
				cout << "Error code " << i << endl;
			}
		}
   }

	// get memory for color image
	win.pvRGB32Image = DepictCreateImageDestinationEx(m_ulImageWidth,
		m_ulImageHeight,
		DepictFormatRGB32);

	USEWIN2 win2.pvRGB32Image = DepictCreateImageDestinationEx(m_ulImageWidth,m_ulImageHeight,DepictFormatRGB32);


   //JDImage im;
	im.resize(m_ulImageWidth, m_ulImageHeight);
	im2.resize(m_ulImageWidth, m_ulImageHeight);
	//im.pixels=(long*)pvRGB32Image;
	
	/*
	DepictConvertBayerToRGB32(m_pvImage, pvRGB32Image, 0,0,m_ulImageWidth,
		m_ulImageHeight);

	unsigned char *capIm=(unsigned char *)pvRGB32Image;
	unsigned long buf;
	unsigned char *bufc= (unsigned char *)&buf;

	for (int y=0;y<m_ulImageHeight*m_ulImageWidth;y++) {
		bufc[0]=capIm[4*y+2];
		bufc[1]=capIm[4*y+1];
		bufc[2]=capIm[4*y+0];
		bufc[3]=255;
		im.pixels[y]=buf;
	}
	*/

   //readJDImage("testingImage.bmp",im); 

   if (im.xsize > 0 || im.ysize>0)
    {
      win.attachImage(im);
		USEWIN2 win2.attachImage(im2);
		cout << "Image Size: " << im.xsize << " " << im.ysize << endl;

	  //ImageWin win2;
	  //win2.attachImage(im);
	 // RedWin rw;
     // RedWin rw2;
		//glutPostRedisplay();

      glutMainLoop();
    }


/*	if(m_pvImage != NULL)
   {
      DepictFreeImageDestination(m_pvImage);
      m_pvImage = NULL;
   }*/

   // close camera
   if(m_hCamera != DEPICT_INVALID_HANDLE)
   {
      DepictCloseDevice(m_hCamera);
      m_hCamera = DEPICT_INVALID_HANDLE;
   }



}

#endif

