#include <stdlib.h>
#include <stdio.h>
#include "CLO.h"
#include <string.h>

#ifdef WIN32
#include "getopt.h"
#pragma warning (disable : 4244 4800 4018)
#endif 


// Static Class Variables
CLO* CLO::clolist[MAXOPTIONS];
int  CLO::clolistlen=0;
char CLO::optionString[1024];
bool CLO::silentMa=false;
int CLO::argcLeft=0;
char ** CLO::argvLeft=NULL;

CLO::CLO()
{
}

/***************************************************************************
                              C L O : : C L O 
***************************************************************************/
CLO::CLO(char *option, char *descript)
{
	char opt1[255], opt2[255];
	opt1[0]=option[0];
	opt1[1]='\0';
	strcpy(opt2,option+1);

	init(opt1,opt2,descript);
}


void CLO::init(char *option, char *opt_param, char *descript)
{
  requiredOption=false;
  requiredOptionFound=false;
  optionarg=1; // does this option have arguments?
  //optionchar=option[0]; // which character is this option
  strcpy(optionchar255,option);
  verifyUniqueOptionChar();
  strcpy(optionname,opt_param);
  // printf ("Addding %c %s %d\n",optionchar,optionname, clolistlen);
  strcpy(optiondescript,descript);
  clolist[clolistlen]=this; // add to the list of current options
  clolistlen++;

  //puts ("creating option");
}

void CLO::allowRemainingArgs()
{
	silentMa=true;
}

int CLO::argc()
{
	return argcLeft;
}

char **CLO::argv()
{
	return argvLeft;
}

/***************************************************************************
           C L O : : V E R I F Y U N I Q U E O P T I O N C H A R 
***************************************************************************/
void CLO::verifyUniqueOptionChar()
{
  // Get the list of all options used so far
  
	for (int i=0; i< clolistlen ; i++) {
		if (strcmp(optionchar255,clolist[i]->optionchar255)==0) {
			fprintf (stderr,"\nCLO ERR: Attempt to specify an option twice! <%s>\n\n",optionchar255);
		}
	}

/*	char *optstr=getOptionString();

  while (strchr(optstr,optionchar)!=NULL)
    {
      optionchar++;
      if (optionchar=='z'+1 || optionchar=='Z'+1)
	optionchar='A';
    }

  */
}

/***************************************************************************
                        C L O : : S H O W U S A G E 
***************************************************************************/
void CLO::showusage(char *execname)
{
  int i;

  printf("\nUsage: %s [options]\n\n",execname);
  
  // Print required options first
  for ( i=0;i<clolistlen;i++) {
    
	if (clolist[i]->requiredOption) {
	   if (clolist[i]->optionarg) {
	  	  printf(" -%s <%s> : <REQUIRED OPTION> \n     %s\n\n",clolist[i]->optionchar255,
	  		clolist[i]->optionname,
			clolist[i]->optiondescript);
	   } else {
		  // this should never be true
	   }
       
	}
  }

  // Now all optional options
  for ( i=0;i<clolistlen;i++) {
    
	if (!clolist[i]->requiredOption) {
	   if (clolist[i]->optionarg) {
	  	  printf(" -%s <%s> : <Default=%s> \n     %s\n\n",clolist[i]->optionchar255,
	  		clolist[i]->optionname,clolist[i]->optiondefaultval,
			clolist[i]->optiondescript);
	   } else {
		  printf(" -%s \n     %s\n\n",clolist[i]->optionchar255,
		    clolist[i]->optiondescript); 
	   }
       
	}
  }

  printf("\n");
  exit(2);
}

/***************************************************************************
                        C L O : : P A R S E A R G S 
***************************************************************************/
int CLO::parseArgs(int argc, char **argv)

{
  int curarg=1;
  bool done=false;

  while (curarg < argc && !done)
    {
	  
	  char *thisarg=argv[curarg];

      int foundoption=0;

	  // If this looks like its an option
      if (strlen(thisarg)>1 && thisarg[0]=='-') {

		 // Loop over known options
		 for (int i=0;i<clolistlen;i++) {

			// If this matches this option (skipping the '-') 
			if (strcmp(thisarg+1,clolist[i]->optionchar255)==0) {

				// Mark the option
				clolist[i]->requiredOptionFound=true;

				// If an option value should exist
				if (clolist[i]->optionarg) {

					// Make sure the arg is actually there
					if (curarg+1 < argc) {
						clolist[i]->setOption(argv[curarg+1]);
						foundoption=1;
						curarg++;
					} else {
						showusage(argv[0]);
					}

				} else {  // flag option

					clolist[i]->setOption(argv[curarg+1]);
					foundoption=1;

				}

			}
		 }
	  }

      if(!foundoption ) {
		  if (silentMa) {
			  done=true;
		  } else {
			  fprintf(stderr,"\nCLO ERR : Unknown option <%s> \n\n",thisarg);
			  showusage(argv[0]);
		  }
	  } else {
		  curarg++;
	  }
    }

  // Did we find all required options
  for (int i=0; i< clolistlen ; i++) {
	  if (clolist[i]->requiredOption && !clolist[i]->requiredOptionFound) {
		fprintf(stderr,"\nCLO ERR : Missing required option <%s> \n\n",clolist[i]->optionchar255);
		showusage(argv[0]);

	  }
  }

  // Set up the left over variables
  argcLeft=argc-curarg+1;
  argvLeft=new char* [argcLeft];
  argvLeft[0]=argv[0];

  //printf ("argcleft %d\n",argcLeft);

  {
  for (int i=1; i<argcLeft; i++) {
	argvLeft[i]=argv[i+curarg-1];
	//printf ("copying arg %s\n",argvLeft[i]);
  }
  }



  /* // THE OLD WAY
  getOptionString();

  extern char *optarg;
  extern int optind;
  int c; 
  int i; 
  

  while ((c = getopt(argc, argv, optionString)) != -1)
    {
      int foundoption=0;
      for ( i=0;i<clolistlen;i++)
	{
	  if (clolist[i]->optionchar==c)
	    {
	      clolist[i]->setOption(optarg);
	      foundoption=1;
	    }
	}
       
      if(!foundoption)
	showusage(argv[0]);
    }
  */

//  return optind;
  return 1;

}

/***************************************************************************
                 * C L O : : G E T O P T I O N S T R I N G 
***************************************************************************/
/*char *CLO::getOptionString()
{
  int i;
  int count=0;
  optionString[0]='\0';
  
  for ( i=0;i<clolistlen;i++,count++)
    {
      optionString[count]=clolist[i]->optionchar;
      if (clolist[i]->optionarg)
	{
	  count++;
	  optionString[count]=':';
	}
    }
  optionString[count]='\0';
  return optionString;
}
*/


// float
CLOfloat::CLOfloat(RequiredType required, char *option, char *opt_param, char *descript)
{
	init(option,opt_param,descript);
	requiredOption=true;
}
CLOfloat::CLOfloat(float nval, char *option, char *opt_param, char *descript)
{
	init(option,opt_param,descript);
	val=nval;
	sprintf(optiondefaultval,"%f", nval);
}
CLOfloat::CLOfloat(float nval, char *option, char *descript):CLO(option,descript)
{val=nval; sprintf(optiondefaultval,"%f",nval);}
float CLOfloat::operator=(float foo) {val=foo; return val;}
float CLOfloat::operator=(CLOfloat &foo) {val=foo.val; return val;}
CLOfloat::operator float() {return val;}
void CLOfloat::setOption(char *optarg) {val=atof(optarg);}

// int
CLOint::CLOint(RequiredType required, char *option, char *opt_param, char *descript)
{
	init(option,opt_param,descript);
	requiredOption=true;
}
CLOint::CLOint(int nval, char *option, char *opt_param, char *descript)
{
	init(option,opt_param,descript);
	val=nval;
	sprintf(optiondefaultval,"%d", nval);
}
CLOint::CLOint(int nval, char *option, char *descript=""):CLO(option,descript)
{val=nval; sprintf(optiondefaultval,"%d",nval);}
int CLOint::operator=(int foo) {val=foo; return val;}
int CLOint::operator=(CLOint &foo) {val=foo.val; return val;}
CLOint::operator int() {return val;}
void CLOint::setOption(char *optarg) {val=atoi(optarg);}

//char255
CLOchar255::CLOchar255(RequiredType required, char *option, char *opt_param, char *descript)
{
	init(option,opt_param,descript);
	requiredOption=true;
}
CLOchar255::CLOchar255(char * nval, char *option, char *opt_param, char *descript)
{
	init(option,opt_param,descript);
	strcpy(val,nval);
	strcpy(optiondefaultval,nval);
}
CLOchar255::CLOchar255(char * nval, char *option, char *descript=""):CLO(option,descript)
{strcpy(val,nval);strcpy(optiondefaultval,nval);}
CLOchar255::operator char*() {return val;}
void CLOchar255::setOption(char *optarg) {strcpy(val,optarg);}

//flag

CLOflag::CLOflag(char *option, char *descript="")
{
	init(option,"",descript);
	val=0; 
	optionarg=0;
}
int CLOflag::operator=(int foo) {val=foo; return val;}
int CLOflag::operator=(CLOflag &foo) {val=foo.val; return val;}
//CLOflag::operator int() {return val;}
CLOflag::operator bool() {return val;}
void CLOflag::setOption(char *optarg) {val=1;}
 
