// ExAvi.cpp
//
// Copyright  1998-2000 Microsoft Corporation, All Rights Reserved
//
//////////////////////////////////////////////////////////////////////

#include "VisWin.h"
#include <VisCore.h>
#if VIS_INTERNAL_RELEASE
    #include <VisImageProc.h>
#endif // VIS_INTERNAL_RELEASE
#include "VisMemoryChecks.h"
#include "ImageWin.h"
#include <iostream.h>
#include "JDImage.h"

class FindFiles 
{
public:
    FindFiles()  { m_hFound = 0;};
    ~FindFiles() { Close();};

    BOOL Open( const char* pImageNames )
    {    
        m_szPrefix[0] = '\0';

        m_hFound = FindFirstFile(pImageNames, &m_FindFileData);
        if (m_hFound == INVALID_HANDLE_VALUE)
        {
            return FALSE;
        }

        // If we are reading files from a directory other than the current one,
        // save the directory name.
        if (strchr(pImageNames, '\\') != NULL)
        {
            strcpy(m_szPrefix, pImageNames);
            char* ptr = strrchr(m_szPrefix,'\\');
            ptr[1] = '\0';
        }

        sprintf(m_szFull, "%s%s", m_szPrefix, m_FindFileData.cFileName);
        return TRUE;
    }

    void FileName( char** ppName, char** ppExt )
    {
        *ppName = m_szFull;
        *ppExt  = strrchr( m_szFull, '.' );
    }

    BOOL Next()
    {
        if ( !FindNextFile(m_hFound, &m_FindFileData) )
            return FALSE;

        sprintf(m_szFull, "%s%s", m_szPrefix, m_FindFileData.cFileName);
        return TRUE;
    }

    void Close()
    {
        if ( m_hFound )
        {
            FindClose(m_hFound);
            m_hFound = 0;
        }
    }

private:

    WIN32_FIND_DATA  m_FindFileData;
    HANDLE           m_hFound;
    char             m_szPrefix[MAX_PATH];
    char             m_szFull[MAX_PATH];
};

class FrameSequencer
{
public:
    FrameSequencer()  { m_pStream = NULL;};
    ~FrameSequencer() { Close();};

    BOOL Initialize( const char* pImageNames)
    {   
        Close();
        return m_ff.Open(pImageNames);
    }

    void Close()
    {
        m_curframe = 0;
        m_bDone    = FALSE;
        if ( m_pStream )
        {
            m_pStream->Release();
            m_pStream = NULL;
        }
        m_ff.Close();
    }

    CVisRGBAByteImage& Next()
    {
        if ( !m_bDone )
        {
            readagain:
            if ( m_pStream )
            {
                if ( m_curframe < m_pStream->FrameCount() )
                {
                    m_curimage.SetStreamHandler(m_pStream,m_curframe);
                    m_curimage.ReadFile();
                    m_curframe++;
                    return m_curimage;
                }
                else if ( !m_ff.Next() )
                {
                    m_bDone = TRUE;
                    return m_curimage;
                }
            }

            char *pname, *pext;
            m_ff.FileName(&pname, &pext);

            if ( pext && stricmp(pext,".avi") == 0 )
            {
                if ( m_pStream )
                    m_pStream->Release();
                m_pStream = OpenStreamHandler(pname, OF_READ);
                m_curframe = 0;
                goto readagain;
            }
            else
            {
                m_curimage.ReadFile(pname);
            }
        }

        if ( !m_ff.Next() )
            m_bDone = TRUE;

        return m_curimage;
    }

private:
    FindFiles          m_ff;
    CVisStreamHandler* m_pStream;
    UINT               m_curframe;
    CVisRGBAByteImage  m_curimage;
    BOOL               m_bDone;
};

static BOOL AddImages(CVisRGBAByteSequence& seq, const char* pImageNames)
{
    FindFiles ff;

    if ( !ff.Open(pImageNames) )
        return FALSE;

    UINT frstpos = seq.Length();
    while (1)
    {
        char *pname, *pext;
        ff.FileName(&pname, &pext);
        if ( pext && stricmp(pext,".avi") == 0 )
        {
            seq.AppendStream(pname);
        }
        else
        {
            seq.AddFile(pname);
        }

        if ( !ff.Next() )
            break;
    }
    ff.Close();

    // XXX mattu need to do this:
    //SortFileNames( seq, frstpos, seq.Length()-frstpos );

    return TRUE;
}

enum MovieStatus {none, read, write};
class MVid {
public:
	CVisStreamHandler* pStrmHandler; // Used for writing
	CVisRGBAByteSequence seq; // Used for reading
	MovieStatus status; // Open for reading/writing/nothing

	MVid():status(none),pStrmHandler(NULL){}
};

void movieOpen(const char *filename, MVid &movie);
void movieOpen(const char *filename, MVid &movie)
{
	// Set sequence options
    int eopts = movie.seq.SequenceOptions();
    eopts &= ~(evissequenceMaskDelayRead|evissequenceMaskLimitMemoryUsage);
    eopts |=  evissequenceDelayReadUntilNeeded|evissequenceLimitMemoryUsage;
    movie.seq.SetSequenceOptions(eopts);

	// Load the movie
	AddImages(movie.seq, filename);

	movie.status=read;
}

void movieCreate(const char *filename, MVid &movie, short xsize, short ysize);
void movieCreate(const char *filename, MVid &movie, short xsize=0, short ysize=0)
{
	bool dialog=FALSE; // Pop up the compression dialog?

    movie.pStrmHandler = OpenStreamHandler(filename, 
                                             OF_READWRITE | OF_CREATE,
                                             32/*bpp*/, dialog );
	movie.status=write;
}

void moviePutFrameCV(MVid &movie, short framenum, CVisRGBAByteImage &frame);
void moviePutFrameCV(MVid &movie, short framenum, CVisRGBAByteImage &frame)
{
	if (movie.status!=write) puts ("Movie not open for writing");
	movie.pStrmHandler->WriteFrame(frame, framenum);
}


void movieGetFrameCV(MVid &movie, short framenum, CVisRGBAByteImage &frame);
void movieGetFrameCV(MVid &movie, short framenum, CVisRGBAByteImage &frame)
{
	if (movie.status!=read) puts ("Movie not open for reading");
	puts ("About to read frame");
	frame = movie.seq[framenum];
	puts ("Just read frame");
}

void CVisRGBA_to_JDImage(CVisRGBAByteImage &in, JDImage &out);
void CVisRGBA_to_JDImage(CVisRGBAByteImage &in, JDImage &out)
{
	out.resize(in.Width(), in.Height());
	for (int x=0;x<in.Width(); x++)
		for (int y=0;y<in.Height(); y++)
		{
			out.r(x,y)=in.Pixel(x,y).R();
			out.g(x,y)=in.Pixel(x,y).G();
			out.b(x,y)=in.Pixel(x,y).B();
			out.a(x,y)=in.Pixel(x,y).A();
		}

}

void JDImage_to_CVisRGBA(JDImage &in, CVisRGBAByteImage &out);
void JDImage_to_CVisRGBA(JDImage &in, CVisRGBAByteImage &out)
{
	CVisRGBAByteImage tmp(in.xsize, in.ysize);
	out=tmp;
	assert (out.Width()==in.xsize && out.Height()==in.ysize);
	for (int x=0;x<out.Width(); x++)
		for (int y=0;y<out.Height(); y++)
		{
			out.Pixel(x,y).SetR(in.r(x,y));
			out.Pixel(x,y).SetG(in.g(x,y));
			out.Pixel(x,y).SetB(in.b(x,y));
			out.Pixel(x,y).SetA(in.a(x,y));
		}

}

void moviePutFrame(MVid &movie, short framenum, JDImage &framejd);
void moviePutFrame(MVid &movie, short framenum, JDImage &framejd)
{
	if (movie.status!=write) puts ("Movie not open for writing");

	CVisRGBAByteImage frame;
	JDImage_to_CVisRGBA(framejd, frame);
	movie.pStrmHandler->WriteFrame(frame, framenum);
}

void movieGetFrame(MVid &movie, short framenum, JDImage &frame);
void movieGetFrame(MVid &movie, short framenum, JDImage &frame)
{
	puts ("About to read JDframe");
	
	CVisRGBAByteImage im;
	movieGetFrameCV(movie, framenum, im);
	CVisRGBA_to_JDImage(im, frame);

	puts ("Just read JDframe");
}

void movieClose(MVid &movie);
void movieClose(MVid &movie)
{
	if (movie.pStrmHandler!=NULL) {
		movie.pStrmHandler->Release();
	}
	movie.status=none;
}

short movieNumFrames(MVid &movie)
{
	return movie.seq.Length();
}

void Usage(const char* progname)
{
#if VIS_INTERNAL_RELEASE
    fprintf(stderr,"%s: [-range first last] [-bpp {24|32}] [-warp k1 k2] [-aud] [-cath] input1 [input2 input3 ...] output.avi|output%%d.{bmp,ppm,...}\n",progname);
#else // VIS_INTERNAL_RELEASE
    fprintf(stderr,"%s: [-range first last] [-bpp {24|32}] [-aud] [-cath] input1 [input2 input3 ...] output.avi|output%%d.{bmp,ppm,...}\n",progname);
#endif // VIS_INTERNAL_RELEASE
    exit(-1);
}


class MovieWin : ImageWin {
public:
	int curFrameNum;
	MVid movie;
	JDImage curFrame;

	MovieWin() {curFrameNum=0;  setIdleInterval(250);}

	void idle() {
		curFrameNum = (curFrameNum+1)%movieNumFrames(movie);
		movieGetFrame(movie, curFrameNum, curFrame);
		attachImage(curFrame);
	}
};


int main(int argc, const char *argv[])
{
    if ( argc < 3 )
        Usage(argv[0]);

	MovieWin mwin;
	movieOpen(argv[1],mwin.movie);

	MVid outmovie;

       // CVisStreamHandler* pStrmHandler = NULL;
         //   pStrmHandler = OpenStreamHandler(argv[2], 
          //                                   OF_READWRITE | OF_CREATE,
//                                             32/*bpp*/, dialog );
		movieCreate(argv[2],outmovie);


        CVisRGBAByteImage outimg;
		JDImage outimgJD;
            int len=mwin.movie.seq.Length();

			for (int i=0; i<mwin.movie.seq.Length() ; i++) {
     
				movieGetFrame(mwin.movie,i,outimgJD);
				
				moviePutFrame(outmovie,i,outimgJD);

                fprintf(stderr,"done writing ");
                        fprintf(stderr," frame # %d\n",i);                   
           
			}

        //pStrmHandler->Release();
		movieClose(outmovie);

	puts ("Calling glut main loop");
	glutMainLoop();

    return 0;
}
