
#include "vidInterfaceVisionSDK.h"

#include <iostream.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>


class FindFiles 
{
public:
    FindFiles()  { m_hFound = 0;};
    ~FindFiles() { Close();};

    BOOL Open( const char* pImageNames )
    {    
        m_szPrefix[0] = '\0';

        m_hFound = FindFirstFile(pImageNames, &m_FindFileData);
        if (m_hFound == INVALID_HANDLE_VALUE)
        {
            return FALSE;
        }

        // If we are reading files from a directory other than the current one,
        // save the directory name.
        if (strchr(pImageNames, '\\') != NULL)
        {
            strcpy(m_szPrefix, pImageNames);
            char* ptr = strrchr(m_szPrefix,'\\');
            ptr[1] = '\0';
        }

        sprintf(m_szFull, "%s%s", m_szPrefix, m_FindFileData.cFileName);
        return TRUE;
    }

    void FileName( char** ppName, char** ppExt )
    {
        *ppName = m_szFull;
        *ppExt  = strrchr( m_szFull, '.' );
    }

    BOOL Next()
    {
        if ( !FindNextFile(m_hFound, &m_FindFileData) )
            return FALSE;

        sprintf(m_szFull, "%s%s", m_szPrefix, m_FindFileData.cFileName);
        return TRUE;
    }

    void Close()
    {
        if ( m_hFound )
        {
            FindClose(m_hFound);
            m_hFound = 0;
        }
    }

private:

    WIN32_FIND_DATA  m_FindFileData;
    HANDLE           m_hFound;
    char             m_szPrefix[MAX_PATH];
    char             m_szFull[MAX_PATH];
};

/*
class FrameSequencer
{
public:
    FrameSequencer()  { m_pStream = NULL;};
    ~FrameSequencer() { Close();};

    BOOL Initialize( const char* pImageNames)
    {   
        Close();
        return m_ff.Open(pImageNames);
    }

    void Close()
    {
        m_curframe = 0;
        m_bDone    = FALSE;
        if ( m_pStream )
        {
            m_pStream->Release();
            m_pStream = NULL;
        }
        m_ff.Close();
    }

    CVisRGBAByteImage& Next()
    {
        if ( !m_bDone )
        {
            readagain:
            if ( m_pStream )
            {
                if ( m_curframe < m_pStream->FrameCount() )
                {
                    m_curimage.SetStreamHandler(m_pStream,m_curframe);
                    m_curimage.ReadFile();
                    m_curframe++;
                    return m_curimage;
                }
                else if ( !m_ff.Next() )
                {
                    m_bDone = TRUE;
                    return m_curimage;
                }
            }

            char *pname, *pext;
            m_ff.FileName(&pname, &pext);

            if ( pext && stricmp(pext,".avi") == 0 )
            {
                if ( m_pStream )
                    m_pStream->Release();
                m_pStream = OpenStreamHandler(pname, OF_READ);
                m_curframe = 0;
                goto readagain;
            }
            else
            {
                m_curimage.ReadFile(pname);
            }
        }

        if ( !m_ff.Next() )
            m_bDone = TRUE;

        return m_curimage;
    }

private:
    FindFiles          m_ff;
    CVisStreamHandler* m_pStream;
    UINT               m_curframe;
    CVisRGBAByteImage  m_curimage;
    BOOL               m_bDone;
};
*/

static BOOL AddImages(CVisRGBAByteSequence& seq, const char* pImageNames)
{
    FindFiles ff;

    if ( !ff.Open(pImageNames) )
        return FALSE;

    UINT frstpos = seq.Length();
    while (1)
    {
        char *pname, *pext;
        ff.FileName(&pname, &pext);
        if ( pext && stricmp(pext,".avi") == 0 )
        {
            seq.AppendStream(pname);
        }
        else
        {
            seq.AddFile(pname);
        }

        if ( !ff.Next() )
            break;
    }
    ff.Close();

    // XXX mattu need to do this:
    //SortFileNames( seq, frstpos, seq.Length()-frstpos );

    return TRUE;
}

static bool fileExists(const char *filename) 
{
    struct stat st;

    if (stat(filename, &st) < 0) {
        if (errno != ENOENT)
            // stat() failed for a reason other than 
            // "file doesn't exist"
            perror(filename);

        return false;
    }
    else
        return true;
}

void movieOpen(const char *filename, MVid &movie)
{
	if (!fileExists(filename)) {
		cerr << "Movie file to open does not exist: " << filename << endl;
		return;
	}

	// Set sequence options
    int eopts = movie.seq.SequenceOptions();
    eopts &= ~(evissequenceMaskDelayRead|evissequenceMaskLimitMemoryUsage);
    eopts |=  evissequenceDelayReadUntilNeeded|evissequenceLimitMemoryUsage;
    movie.seq.SetSequenceOptions(eopts);

	// Load the movie
	AddImages(movie.seq, filename);

	movie.status=read;
}
 
void movieCreate(const char *filename, MVid &movie, short xsize, short ysize)
{
	bool dialog=FALSE; // Pop up the compression dialog?

    movie.pStrmHandler = OpenStreamHandler(filename, 
                                             OF_READWRITE | OF_CREATE,
                                             32/*bpp*/, dialog );
	movie.status=write;
}

void movieCreateWithCompressDialog(const char *filename, MVid &movie, short xsize, short ysize)
{
	bool dialog=TRUE; // Pop up the compression dialog?

    movie.pStrmHandler = OpenStreamHandler(filename, 
                                             OF_READWRITE | OF_CREATE,
                                             32/*bpp*/, dialog );
	movie.status=write;
}

void moviePutFrameCV(MVid &movie, short framenum, CVisRGBAByteImage &frame)
{
	if (movie.status!=write) puts ("Movie not open for writing");
	movie.pStrmHandler->WriteFrame(frame, framenum);
}


void movieGetFrameCV(MVid &movie, short framenum, CVisRGBAByteImage &frame)
{
	if (movie.status!=read) puts ("Movie not open for reading");
	//puts ("About to read frame");
	frame = movie.seq[framenum];
//	puts ("Just read frame");
}

void CVisRGBA_to_JDImage(CVisRGBAByteImage &in, JDImage &out)
{
	out.resize(in.Width(), in.Height());

	int width=in.Width();
	int height=in.Height();

/*

  //This is the old safe way, but for speed reasons we switched

	for (int x=0;x<width; x++)
		for (int y=0;y<height; y++)
		{
			out.r(x,y)=in.Pixel(x,y).R();
			out.g(x,y)=in.Pixel(x,y).G();
			out.b(x,y)=in.Pixel(x,y).B();
			out.a(x,y)=in.Pixel(x,y).A();
		}

*/	

	// New fast copy
	assert(width==in.Right());
	assert(height==in.Bottom());
	assert(in.Top()==0);
	assert(in.Left()==0);
	for (int y = 0; y < height;y++)
	{
		unsigned char *Iin = (unsigned char *) (in.RowPointer(y));
		unsigned char *Iout = (unsigned char *) (&out.pix(0,y));
		for (int x = 0; x < width*4;x=x+4) {
			Iout[x+0]=Iin[x+2];
			Iout[x+1]=Iin[x+1];
			Iout[x+2]=Iin[x+0];
			Iout[x+3]=Iin[x+3];
		}
	}


}

void JDImage_to_CVisRGBA(JDImage &in, CVisRGBAByteImage &out)
{
	CVisRGBAByteImage tmp(in.xsize, in.ysize);
	out=tmp;
	assert (out.Width()==in.xsize && out.Height()==in.ysize);
	int width=out.Width();
	int height=out.Height();

	/*
	for (int x=0;x<out.Width(); x++)
		for (int y=0;y<out.Height(); y++)
		{
			out.Pixel(x,y).SetR(in.r(x,y));
			out.Pixel(x,y).SetG(in.g(x,y));
			out.Pixel(x,y).SetB(in.b(x,y));
			out.Pixel(x,y).SetA(in.a(x,y));
		}
	*/
	for (int y = 0; y < height;y++)
	{
		unsigned char *Iout = (unsigned char *) (out.RowPointer(y));
		unsigned char *Iin = (unsigned char *) (&in.pix(0,y));
		for (int x = 0; x < width*4;x=x+4) {
			Iout[x+0]=Iin[x+2];
			Iout[x+1]=Iin[x+1];
			Iout[x+2]=Iin[x+0];
			Iout[x+3]=Iin[x+3];
		}
	}

}

void moviePutFrame(MVid &movie, short framenum, JDImage &framejd)
{
	if (movie.status!=write) puts ("Movie not open for writing");

	CVisRGBAByteImage frame;
	JDImage_to_CVisRGBA(framejd, frame);
	movie.pStrmHandler->WriteFrame(frame, framenum);
}

void movieGetFrame(MVid &movie, short framenum, JDImage &frame)
{
//	puts ("About to read JDframe");

	CVisRGBAByteImage im;
	movieGetFrameCV(movie, framenum, im);
	CVisRGBA_to_JDImage(im, frame);

//	puts ("Just read JDframe");
}  

void movieClose(MVid &movie)
{
	if (movie.pStrmHandler!=NULL) {
		movie.pStrmHandler->Release();
	}
	movie.status=none;
}

short movieNumFrames(MVid &movie)
{
	return movie.seq.Length();
}

void movieXYSize(MVid &movie, short &x, short &y)
{
	JDImage tmp;
	movieGetFrame(movie,1,tmp);
	x=tmp.xsize;
	y=tmp.ysize;
}

