#ifndef _ARRAY_H_
#define _ARRAY_H_

#include <cstdlib>
#include <cstdio>
#include <new>

#include "error.h"

template <class T> class Array {
    public:
        Array() 
        {
            buffer = NULL;
            capacity = size = 0;
        }

        Array(const Array<T> &other)
        {
            buffer = NULL;
            capacity = size = 0;
            *this = other;
        }

        virtual ~Array()
        {
            Reserve(0);
        }

        Array<T> &operator=(const Array<T> &other)
        {
            DecreaseSize(size);
            if (!Reserve(other.size)) 
                assertf(0, ("Unable to resize buffer"));
            for (int i = 0; i < other.size; i++)
                IncreaseSize(1, other.buffer[i]); 
            return *this;
        }

        virtual T& operator[](int index)
        {
            return buffer[index];
        }

        virtual const T& operator[](int index) const
        {
            return buffer[index];
        }

        virtual int PushBack(const T& element = T())
        {
            if (size >= capacity && 
                    !(Reserve(capacity > 0 ? 2*capacity : 2) 
                        || Reserve(size+1))) 
                return 0;
            IncreaseSize(1, element);
            return 1;
        }

        virtual int PushBack(const Array<T> &other)
        {
            if (size + other.size > capacity && !Reserve(size + other.size))
                return 0;
            for (int i = 0; i < other.size; i++) 
                IncreaseSize(1, other.buffer[i]);
            return 1;
        }

        virtual void Empty(void)
        {
			DecreaseSize(size);
		}

        virtual int Reserve(int _capacity)
        {
            if (_capacity > capacity) {
                if (buffer) {
                    T *temp = (T*) realloc(buffer, _capacity*sizeof(T));
                    if (!temp) return 0;
                    buffer = temp;
                    capacity = _capacity;
                    return 1;
                } else {
                    buffer = (T*) malloc(_capacity*sizeof(T));
                    if (!buffer) return 0;
                    capacity = _capacity;
                    return 1;
                }
            } else {
                if (buffer) {
                    if (size > _capacity) {
                        DecreaseSize(size - _capacity);
                        if (_capacity > 0) {
                            T *temp = (T*) realloc(buffer, _capacity*sizeof(T));
                            if (!temp) return 1;
                            buffer = temp;
                        } else {
                            free(buffer);
                            buffer = NULL;
                        }
                        capacity = _capacity;
                        return 1;
                    }
                } else return 1;
            }
            return 1;
        }

        virtual int Resize(int _size, const T &element = T())
        {
            if (_size > size) {
                if (_size > capacity && !Reserve(_size)) return 0;
                IncreaseSize(_size - size, element);
                return 1;
            } else {
                DecreaseSize(size - _size);
                return 1;
            }
        }

        virtual int GetSize(void) const
        {
            return size;
        }

        virtual int GetCapacity(void) const 
        {
            return capacity;
        }

    private:

        void IncreaseSize(int amount, const T &element)
        {
            while (amount--)
                // allocate at position bufer+size a object of type
                // T calling the copy constructor to get the value
                // from element
                new(buffer+size++)T(element);
        }

        void DecreaseSize(int amount)
        {
			while (amount--) {
				size = size - 1;
                buffer[size].~T();
			}
        }

        int size, capacity;
        T* buffer;

};

#endif
