#ifndef FIXED_H
#define FIXED_H

template <int F> class Fixed {
    public:
        Fixed(int v = 0) { f = (long int) (v  << F); }
        Fixed(double v) { f = (long int) (v * (1 << F)); }

        operator double() { return (double) (f) / (1 << F); }

        Fixed<F> operator+(const Fixed<F> &g) 
        { 
            Fixed<F> r;
            r.f = f + g.f; 
            return r;
        }
        Fixed<F> operator+(const double &g) { return (*this) + (Fixed<F>) g; }
        Fixed<F> operator+(const int &g) { return (*this) + (Fixed<F>) g; }

        Fixed<F> operator-(const Fixed<F> &g) 
        { 
            Fixed<F> r;
            r.f = f - g.f; 
            return r;
        }
        Fixed<F> operator-(const double &g) { return (*this) - (Fixed<F>) g; }
        Fixed<F> operator-(const int &g) { return (*this) - (Fixed<F>) g; }

        Fixed<F> operator*(const Fixed<F> &g) 
        { 
            Fixed<F> r;
            r.f = (f * g.f) >> F; 
            return r;
        }
        Fixed<F> operator*(const double &g) { return (*this) * (Fixed<F>) g; }
        Fixed<F> operator*(const int &g) { return (*this) * (Fixed<F>) g; }

        Fixed<F> operator/(const Fixed<F> &g) 
        { 
            Fixed<F> r; 
            r.f = (f << F) / g.f; 
            return r;
        }
        Fixed<F> operator/(const double &g) { return (*this) * (Fixed<F>) g; }
        Fixed<F> operator/(const int &g) { return (*this) * (Fixed<F>) g; }

        Fixed<F> &operator+=(const Fixed<F> &g)
        {
            f += g.f;
            return *this;
        }

        Fixed<F> &operator-=(const Fixed<F> &g)
        {
            f -= g.f;
            return *this;
        }

        Fixed<F> &operator*=(const Fixed<F> &g)
        {
            f *= g.f;
            f >>= F;
            return *this;
        }

        Fixed<F> &operator/=(const Fixed<F> &g)
        {
            f <<= F;
            f /= g;
            return *this;
        }

        bool operator==(const Fixed<F> &g)
        {
            return f == g.f;
        }
        bool operator==(const double &g) { return (*this) == (Fixed<F>) g; } 
        bool operator==(const int &g) { return (*this) == (Fixed<F>) g; } 

        bool operator!=(const Fixed<F> &g)
        {
            return f != g.f;
        }
        bool operator!=(const double &g) { return (*this) != (Fixed<F>) g; } 
        bool operator!=(const int &g) { return (*this) != (Fixed<F>) g; } 

        bool operator>(const Fixed<F> &g)
        {
            return f > g.f;
        }
        bool operator>(const double &g) { return (*this) > (Fixed<F>) g; } 
        bool operator>(const int &g) { return (*this) > (Fixed<F>) g; } 

        bool operator>=(const Fixed<F> &g)
        {
            return f >= g.f;
        }
        bool operator>=(const double &g) { return (*this) >= (Fixed<F>) g; } 
        bool operator>=(const int &g) { return (*this) >= (Fixed<F>) g; } 

        bool operator<(const Fixed<F> &g)
        {
            return f < g.f;
        }
        bool operator<(const double &g) { return (*this) < (Fixed<F>) g; } 
        bool operator<(const int &g) { return (*this) < (Fixed<F>) g; } 

        bool operator<=(const Fixed<F> &g)
        {
            return f < g.f;
        }
        bool operator<=(const double &g) { return (*this) <= (Fixed<F>) g; } 
        bool operator<=(const int &g) { return (*this) <= (Fixed<F>) g; } 

    private:
        long int f;
};

template <int F> 
Fixed<F> operator-(const double &f, const Fixed<F> &g)
{ return ((Fixed<F>) f) - g; }
template <int F> 
Fixed<F> operator-(const int &f, const Fixed<F> &g)
{ return ((Fixed<F>) f) - g; }

template <int F> 
Fixed<F> operator+(const double &f, const Fixed<F> &g)
{ return ((Fixed<F>) f) + g; }
template <int F> 
Fixed<F> operator+(const int &f, const Fixed<F> &g)
{ return ((Fixed<F>) f) + g; }

template <int F> 
Fixed<F> operator*(const double &f, const Fixed<F> &g)
{ return ((Fixed<F>) f) * g; }
template <int F> 
Fixed<F> operator*(const int &f, const Fixed<F> &g)
{ return ((Fixed<F>) f) * g; }

template <int F> 
Fixed<F> operator/(const double &f, const Fixed<F> &g)
{ return ((Fixed<F>) f) / g; }
template <int F> 
Fixed<F> operator/(const int &f, const Fixed<F> &g)
{ return ((Fixed<F>) f) / g; }

template <int F> 
bool operator==(const int &f, const Fixed<F> &g)
{ return ((Fixed<F>) f) == g; }
template <int F> 
bool operator==(const double &f, const Fixed<F> &g)
{ return ((Fixed<F>) f) == g; }

template <int F> 
bool operator!=(const int &f, const Fixed<F> &g)
{ return ((Fixed<F>) f) != g; }
template <int F> 
bool operator!=(const double &f, const Fixed<F> &g)
{ return ((Fixed<F>) f) != g; }

template <int F> 
bool operator<(const int &f, const Fixed<F> &g)
{ return ((Fixed<F>) f) < g; }
template <int F> 
bool operator<(const double &f, const Fixed<F> &g)
{ return ((Fixed<F>) f) < g; }

template <int F> 
bool operator<=(const int &f, const Fixed<F> &g)
{ return ((Fixed<F>) f) <= g; }
template <int F> 
bool operator<=(const double &f, const Fixed<F> &g)
{ return ((Fixed<F>) f) <= g; }

template <int F> 
bool operator>(const int &f, const Fixed<F> &g)
{ return ((Fixed<F>) f) > g; }
template <int F> 
bool operator>(const double &f, const Fixed<F> &g)
{ return ((Fixed<F>) f) > g; }

template <int F> 
bool operator>=(const int &f, const Fixed<F> &g)
{ return ((Fixed<F>) f) >= g; }
template <int F> 
bool operator>=(const double &f, const Fixed<F> &g)
{ return ((Fixed<F>) f) >= g; }

#endif
