#include "error.h"
#include "magickimage.h" 

int MagickImage::Load(const char *name)
{
    try { image.read(name); image.flip(); }
    catch (...) { return 0; }
    return 1;
}

int MagickImage::Load(const BufferUchar &buffer)
{
    switch (buffer.GetDimension()) {
        case 1:
            try {
                image.read(buffer.GetWidth(), buffer.GetHeight(), "I", 
                        Magick::CharPixel, buffer.GetBuffer());
                image.flip();
            }
            catch (...) { return 0; }
            return 1;
        case 3:
            try {
                image.read(buffer.GetWidth(), buffer.GetHeight(), "RGB", 
                        Magick::CharPixel, buffer.GetBuffer());
                image.flip();
            }
            catch (...) { return 0; }
            return 1;
        default:
            return 0;
    }
}

int MagickImage::Load(const unsigned char *data, size_t size)
{
    try {
        Magick::Blob blob(data, size);
        image.read(blob);
        image.flip();
    }
    catch (...) { return 0; }
    return 1;
}

int MagickImage::Store(const char *name)
{
    try { image.write(name); }
    catch (...) { return 0; }
    return 1;
}

int MagickImage::Store(BufferUchar &buffer, int mode)
{
    try { 
        switch (mode) {
            case IMAGE_GRAY:
                if (!buffer.Resize(GetWidth(), GetHeight(), 1)) return 0;
                image.type(Magick::GrayscaleType);
                image.write(0, 0, GetWidth(), GetHeight(), "I", 
                    Magick::CharPixel, buffer.GetBuffer()); 
                break;
            case IMAGE_RGB:
                if (!buffer.Resize(GetWidth(), GetHeight(), 3)) return 0;
                image.type(Magick::TrueColorType);
                image.write(0, 0, GetWidth(), GetHeight(), "RGB", 
                    Magick::CharPixel, buffer.GetBuffer()); 
                break;
            default:
                errorf(("Unsupported mode"));
                return 0;
        }
    }
    catch (...) { return 0; }
    return 1;
}

int MagickImage::GetWidth()
{
    return image.columns();
}

int MagickImage::GetHeight()
{
    return image.rows();
}
