#include "glutwm.h"
#include "plotwindow.h"
    
PlotWindow2::
PlotWindow2()
{
    plot.Resize(1);
}

int 
PlotWindow2::
SetData(const Poly2 &_data, int set)
{
    if (plot.GetSize() <= set && !plot.Resize(set+1)) {
        errorf(("Unable to resize plot array"));
        return 0;
    }
    plot[set].data = _data;
    return 1;
}

int 
PlotWindow2::
SetData(const double *_data, int count, int set)
{
    if (plot.GetSize() <= set && !plot.Resize(set+1)) {
        errorf(("Unable to resize plot array"));
        return 0;
    }
    plot[set].data.Empty();
    for (int i = 0; i < count; i++) {
        if (!plot[set].data.PushBack(Vector2(i, _data[i]))) {
            errorf(("Unable add entry"));
            return 0;
        }
    }
    return 1;
}

int 
PlotWindow2::
SetData(const float *_data, int count, int set)
{
    if (plot.GetSize() <= set && !plot.Resize(set+1)) {
        errorf(("Unable to resize plot array"));
        return 0;
    }
    plot[set].data.Empty();
    for (int i = 0; i < count; i++) {
        if (!plot[set].data.PushBack(Vector2(i, _data[i]))) {
            errorf(("Unable add entry"));
            return 0;
        }
    }
    return 1;
}

void 
PlotWindow2::
Empty(void)
{
    for (int i = 0; i < plot.GetSize(); i++)
        plot[i].data.Empty();
}

void 
PlotWindow2::
Color(int set)
{
    float r, g, b;
    switch (set % 6) {
        case 0:
            r = 1; g = 0; b = 0;
            break;
        case 1:
            r = 0; g = 1; b = 0;
            break;
        case 2:
            r = 0; g = 0; b = 1;
            break;
        case 3:
            r = 1; g = 1; b = 0;
            break;
        case 4:
            r = 0; g = 1; b = 1;
            break;
        case 5:
            r = 1; g = 0; b = 1;
            break;
    }
    float d = set/6 + 1;
    r /= d;
    g /= d;
    b /= d;
    glColor3f(r, g, b);
}

Poly2 &
PlotWindow2::
GetData(int set)
{
    return plot[set].data;
}

int 
PlotWindow2::
SetBBox(const BBox2 &_box, int set)
{
    if (plot.GetSize() <= set && !plot.Resize(set+1)) {
        errorf(("Unable to resize plot array"));
        return 0;
    }
    plot[set].box = _box;
    return 1;
}

void 
PlotWindow2::
DrawData(void)
{
    glMatrixMode(GL_PROJECTION);
    // draw stuff
    for (int d = 0; d < plot.GetSize(); d++) {
        BBox2 box = plot[d].box;
        double x1 = box.GetMin()[X]; double x2 = box.GetMax()[X];
        double y1 = box.GetMin()[Y]; double y2 = box.GetMax()[Y];
        // set world to containg bounding box
        glLoadIdentity();
        glOrtho(x1, x2, y1, y2, -1, 1);
        Color(d);
        glPointSize(3);
        glBegin(GL_POINTS);
        for (int i = 0; i < plot[d].data.GetSize(); i++)
            glVertex2d(plot[d].data[i][X], plot[d].data[i][Y]);
        glEnd();
    }
}

void 
PlotWindow2::
DrawFrame(void)
{
    int w = GetWidth(), h = GetHeight();
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(0, w-1, 0, h-1, -1, 1);
    glBegin(GL_LINE_STRIP);
        glVertex2d(0, 0);
        glVertex2d(0, h-1);
        glVertex2d(w-1, h-1);
        glVertex2d(w-1, 0);
        glVertex2d(0, 0);
    glEnd();
}

void 
PlotWindow2::
Display(void)
{
    debugf(("Plot redraw"));
    int w = GetWidth(), h = GetHeight();
    glClearColor(1, 1, 1, 1);
    glClear(GL_COLOR_BUFFER_BIT);
    if (plot.GetSize() < 1) return;
    glShadeModel(GL_FLAT);
    // draw graph
    glViewport(5, 5, w-10, h-10);
    glColor3d(0.75, 0.75, 0.75);
    DrawData();
    glColor3d(0, 0, 0);
    DrawFrame();
}
