#ifndef VOLUME_H
#define VOLUME_H

#include "buffer.h"
    
template <class T> class Volume: public Buffer<T> {
    public:
        int Resize(int width, int height, int dimension, int _length)
        {
            if (Buffer<T>::Resize(width, height, dimension*_length)) {
                length = _length;
                return 1;
            } else return 0;
        }

        template <class U>
        int Store(int frame, Buffer<U> &other)
        {
            if (frame >= length || frame < 0) {
                errorf(("Frame %d is out of range"));
                return 0;
            }
            int w, h, d;
            const T *p = GetBuffer(&w, &h, &d);
            p += frame*d;
            if (!other.Resize(w, h, d)) {
                errorf(("Unable to resize other buffer"));
                return 0;
            }
            U *op = other.GetBuffer();
            for (int i = 0; i < w*h; i++) {
                for (int j = 0; j < d; j++) 
                    Translate(op[j], p[j]);
                op += d;
                p += d*length;
            }
            return 1;
        }

        template <class U>
        int Load(int frame, const Buffer<U> &other)
        {
            if (frame >= length || frame < 0) {
                errorf(("Frame %d is out of range"));
                return 0;
            }
            int w, h, d;
            T *p = GetBuffer(&w, &h, &d);
            p += frame*d;
            int ow, oh, od;
            const U *op = other.GetBuffer(&ow, &oh, &od);
            if (w != ow || h != oh || d != od) {
                errorf(("Frame is %dx%dx%d, expected %dx%dx%d", 
                            ow, oh, od, w, h, d));
                return 0;
            }
            for (int i = 0; i < w*h; i++) {
                for (int j = 0; j < d; j++) 
                    Translate(p[j], op[j]);
                op += d;
                p += d*length;
            }
            return 1;
        }

        T* GetBuffer(int *_width = NULL, int *_height = NULL, 
                int *_dimension = NULL, int *_length = NULL)
        {
            T* data = Buffer<T>::GetBuffer(_width, _height, _dimension);
            if (_dimension) *_dimension /= length;
            if (_length) *_length = length;
            return data;
        }

        const T* GetBuffer(int *_width = NULL, int *_height = NULL, 
                int *_dimension = NULL, int *_length = NULL) const
        {
            T* data = Buffer<T>::GetBuffer(_width, _height, _dimension);
            if (_dimension) *_dimension /= length;
            if (_length) *_length = length;
            return data;
        }

        int GetLength(void) const 
        { return length; }

        int GetDimension(void) const
        { return Buffer<T>::GetDimension() / length; }

    private:
        int length;
};

typedef Volume<Uchar> VolumeUchar;

#endif // VOLUME_H
